@@ -1,4 +1,5 @@
 ;; Copyright (C) 1994 W. Schelter
+;; Copyright (C) 2024 Camm Maguire
 
 ;; This file is part of GNU Common Lisp, herein referred to as GCL
 ;;
@@ -187,9 +188,8 @@
 (defvar *string-streams* (list (make-string-input-stream "") (make-string-input-stream "")))
 
 (defmacro with-tk-command (&body body)
-  `(let (tk-command (*command-strings* *command-strings*))
-     (declare (type string tk-command))
-     (setq tk-command (grab-tk-command))
+  `(let ((tk-command (grab-tk-command)) (*command-strings* *command-strings*))
+     (declare (string tk-command))
      ,@ body))
 
 (defun grab-tk-command( &aux x)
@@ -198,11 +198,11 @@
    ((cdr *command-strings*))
    (t 
     (setq x (list (make-array 70
-			      :element-type 'standard-char
+			      :element-type 'character
 			      :fill-pointer 0 :adjustable t))
 	  )
     (or *command-strings* (error "how??"))
-  
+
     (setq *command-strings* (nconc *command-strings* x))))
   (let ((x (car *command-strings*)))
     (setq  *command-strings* (cdr *command-strings*))
@@ -741,10 +741,9 @@
 		      (setq ans (format nil "~a ~a" ans (car tem))))
 		     (t (error "bad arg"))))
 	    (if ans (setq ans (concatenate 'string "{(" ans ")}")))
-	    (setq command `(lambda ,*percent-symbols-used*
-			     ,command))
+	    (setq command (eval `(lambda ,*percent-symbols-used* ,command)))
 	    (if ans (setq name (concatenate 'string "{"name " " ans"}"))))
-	   (t (setq command `(lambda (&rest *arglist*) ,command))))
+	   (t (setq command (eval `(lambda (&rest *arglist*) ,command)))))
      (setf (aref *call-backs* ind)  command)
      ;; the command must NOT appear as "{[...]}" or it will be eval'd. 
      (cons 'a name)
