@@ -85,15 +85,6 @@ do {								\
  */
 #define SIGPROTV SIGBUS
 
-#ifdef IN_GBC
-#undef MPROTECT_ACTION_FLAGS
-#define MPROTECT_ACTION_FLAGS SA_RESTART|SA_SIGINFO
-#define GET_FAULT_ADDR(sig,code,sv,a) \
- ((siginfo_t *)code)->si_addr
-/*  #define GET_FAULT_ADDR(sig,code,sv,a) \ */
-/*      ((void *)(*((char ***)(&code)))[44]) */
-#endif
-
 /* Begin for cmpinclude */
 #define SGC	/* can mprotect pages and so selective gc will work */
 /* End for cmpinclude */
