@@ -6,7 +6,7 @@
   case X86_64_RELOC_UNSIGNED:		// for absolute addresses
 
      if (ri->r_extern || !ri->r_pcrel) 
-      add_val(q,~0L,ri->r_pcrel ? a-rel : a);
+      store_val(q,~0L,ri->r_pcrel ? a-rel : a);
 
     break; 
   case X86_64_RELOC_GOT_LOAD:		// a MOVQ load of a GOT entry
@@ -16,8 +16,6 @@
     *got=a;
     a=(ul)got;
 
-  case X86_64_RELOC_SIGNED_1:		// for signed 32-bit displacement with -1 addend,
-                                        //   apparently already in *q
   case X86_64_RELOC_SIGNED:		// for signed 32-bit displacement
   case X86_64_RELOC_BRANCH:		// a CALL/JMP instruction with 32-bit displacement
 
