@@ -6,8 +6,6 @@
 /* define to show we included mp.h */
 #define _MP_H  
 
-#define MOST_NEG_FIXNUM (-2147483647-1)
-#define MOST_POSITIVE_FIXNUM (2147483647)
 
 #define MP_ALLOCATED(x) MP(x)->_mp_alloc
 #define MP_SELF(x) MP(x)->_mp_d
@@ -44,8 +42,8 @@ typedef struct
 
 #define MPOP(action,function,x1,x2) \
   do {  \
-   function(MP(big_fixnum2) ,x1,x2); \
-   action maybe_replace_big(big_fixnum2); \
+   function(MP(big_fixnum1) ,x1,x2); \
+   action maybe_replace_big(big_fixnum1); \
   } while(0)
 
 
@@ -56,10 +54,10 @@ typedef struct
       } while(0)
 
 
-#define MYmake_fixnum(action,x) \
-  do{register int CMPt1; \
-   action \
-   ((((CMPt1=(x))+1024)&-2048)==0?small_fixnum(CMPt1):make_fixnum1(CMPt1));}while(0)
+/* #define MYmake_fixnum(action,x) \ */
+/*   do{register int CMPt1; \ */
+/*    action \ */
+/*    ((((CMPt1=(x))+1024)&-2048)==0?small_fixnum(CMPt1):make_fixnum1(CMPt1));}while(0) */
      
 #define ineg(a_) (sizeof(a_)==sizeof(unsigned) ? (unsigned)-(a_) : (unsigned long)-(a_))
 
@@ -75,7 +73,7 @@ typedef struct
 #define subsi(u,a,b) mpz_sub(u,SI_TO_MP(a,big_fixnum1),b)
 #define subis(u,a,b) (b >= 0 ?  mpz_sub_ui(u,a,b) : mpz_add_ui(u,a,ineg(b)))
 #define subss(u,a,b) subis(u,SI_TO_MP(a,big_fixnum1),b)
-#define shifti(u,a,w) (w>=0 ? mpz_mul_2exp(u,a,w) : mpz_fdiv_q_2exp(u,MP(x),ineg(w)))
+#define shifti(u,a,w) (w>=0 ? mpz_mul_2exp(u,a,w) : mpz_fdiv_q_2exp(u,a,ineg(w)))
 
 
 
