@@ -8,7 +8,6 @@
 #define	CHAR_CODE_LIMIT	256	
 #define	READ_TABLE_SIZE CHAR_CODE_LIMIT
 
-void enter_mark_origin() ;
 
 EXTER int *cs_org;     
 EXTER int GBC_enable;
@@ -18,9 +17,7 @@ EXTER object sSAnotify_gbcA;
 
 /* symbols which are not needed in compiled lisp code */
 EXTER int interrupt_flag,interrupt_enable;
-void install_default_signals();
 /* void sigint(),sigalrm(); */
-void segmentation_catcher();
 
 
 EXTER int gc_enabled, saving_system;
@@ -29,7 +26,6 @@ EXTER object lisp_package,user_package;
 EXTER char *core_end;
 EXTER int catch_fatal;
 EXTER long real_maxpage;
-char *getenv();
 EXTER char *this_lisp;
 
 EXTER char stdin_buf[],stdout_buf[];
@@ -95,6 +91,7 @@ EXTER object user_package;
 #define make_macro_function(a_,b_) make_macro_internal(a_,FFN(b_))
 #define make_si_function(a_,b_) make_si_function_internal(a_,FFN(b_))
 #define make_special_form(a_,b_) make_special_form_internal(a_,FFN(b_))
+#define make_si_special_form(a_,b_) make_si_special_form_internal(a_,FFN(b_))
 #define make_macro_function(a_,b_) make_macro_internal(a_,FFN(b_))
 #define make_si_sfun(a_,b_,c_) make_si_sfun_internal(a_,FFN(b_),c_)
 #define STATD static
@@ -106,6 +103,7 @@ EXTER object user_package;
 #define make_macro_function(a_,b_) make_macro_internal(a_,b_)
 #define make_si_function(a_,b_) make_si_function_internal(a_,b_)
 #define make_special_form(a_,b_) make_special_form_internal(a_,b_)
+#define make_si_special_form(a_,b_) make_si_special_form_internal(a_,b_)
 #define make_macro_function(a_,b_) make_macro_internal(a_,b_)
 #define make_si_sfun(a_,b_,c_) make_si_sfun_internal(a_,b_,c_)
 #define STATD
@@ -154,8 +152,6 @@ STATD ret FFN(fname) args
 #endif /* NO_DEFUN */
 
 
-object  type_name();
-object fSincorret_type();
 
 
 #define TYPE_OF(x) type_of(x)
@@ -170,7 +166,6 @@ TS_MEMBER(t0,TS(t1)|TS(t2)|TS(t3)...)
 
 #define ASSURE_TYPE(val,t) if (type_of(val)!=t) TYPE_ERROR(val,type_name(t))
 
-object IisArray();
 
 /* array to which X is has its body displaced */
 #define DISPLACED_TO(x) Mcar(ADISP(x))
@@ -207,16 +202,12 @@ EXTER int eval1 ;
 EXTER bool in_list_flag;
 EXTER object test_function;
 EXTER object item_compared;
-bool (*tf)();
 EXTER object key_function;
-object (*kf)();
-object (*car_or_cdr)();
 
 
 /* string.d */
 EXTER  bool left_trim;
 EXTER bool right_trim;
-int  (*casefun)();
 
 
 /* on most machines this will test in one instruction
@@ -238,10 +229,7 @@ EXTER object sSXB;
 
 #define	inheap(pp)	((char *)(pp) < heap_end)
 
-char *lisp_copy_to_null_terminated();
 
-int
-gcl_init_cmp_anon(void);
 
 #undef SAFE_READ
 #undef SAFE_FREAD
@@ -323,12 +311,24 @@ extern bool writable_malloc;
 
 #include "error.h"
 
+#if __GNU_MP_VERSION > 4 || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
+extern void __gmp_randget_mt ();
+extern void __gmp_randclear_mt ();
+extern void __gmp_randiset_mt ();
+
+typedef struct {void *a,*b,*c,*d;} gmp_randfnptr_t;
+EXTER gmp_randfnptr_t Mersenne_Twister_Generator_Noseed;
+#endif
 
 /* #define BV_BITS CHAR_SIZE */
 /* #define BV_ALLOC (BV_BITS*SIZEOF_LONG) */
 #define BV_BITS (CHAR_SIZE*SIZEOF_LONG)
 #define BV_ALLOC BV_BITS
+#ifdef WORDS_BIGENDIAN
+#define BV_BIT(i) (1L<<(BV_BITS-1-((i)%BV_BITS)))
+#else
 #define BV_BIT(i) (1L<<((i)%BV_BITS))
+#endif
 #define BITREF(x,i) ({ufixnum _i=(i);(BV_BIT(_i)&(x->bv.bv_self[_i/BV_BITS])) ? 1 : 0;})
 #define SET_BITREF(x,i)   ({ufixnum _i=(i);(x->bv.bv_self[_i/BV_BITS]) |= BV_BIT(_i);})
 #define CLEAR_BITREF(x,i) ({ufixnum _i=(i);(x->bv.bv_self[_i/BV_BITS]) &= ~BV_BIT(_i);})
