@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -33,27 +34,10 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 object Vrandom_state;
 
-
-#define BOOLCLR		0
-#define BOOLSET		017
-#define BOOL1		03
-#define BOOL2		05
-#define BOOLC1		014
-#define BOOLC2		012
-#define BOOLAND		01
-#define BOOLIOR		07
-#define BOOLXOR		06
-#define BOOLEQV		011
-#define BOOLNAND	016
-#define BOOLNOR		010
-#define BOOLANDC1	04
-#define BOOLANDC2	02
-#define BOOLORC1	015
-#define BOOLORC2	013
-
 #ifndef PI
 #define PI			3.141592653589793
 #endif
 
-#define MOST_POSITIVE_FIX	2147483647
+#define LOG_WORD_SIZE           (8*SIZEOF_LONG)
+#define MOST_POSITIVE_FIX	((long)((((unsigned long)1)<<(LOG_WORD_SIZE-1))-1))
 #define MOST_NEGATIVE_FIX	( - MOST_POSITIVE_FIX - 1 )
