@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -420,9 +421,10 @@ EXTER long holepage;			/*  hole pages  */
 #define rbgbccount tm_table[t_relocatable].tm_gbccount
 EXTER fixnum new_holepage,starting_hole_div,starting_relb_heap_mult;
 
-EXTER ufixnum recent_allocation,wait_on_abort;
+EXTER ulfixnum cumulative_allocation,recent_allocation;
+EXTER ufixnum wait_on_abort;
 EXTER double gc_alloc_min,mem_multiple,gc_page_min,gc_page_max;
-EXTER bool multiprocess_memory_pool;
+EXTER char *multiprocess_memory_pool;
 
 EXTER char *new_rb_start;		/*  desired relblock start after next gc  */
 EXTER char *rb_start;           	/*  relblock start  */
@@ -470,7 +472,7 @@ ufmax(ufixnum a,ufixnum b) {
 }
 
 INLINE int
-emsg(const char *s,...) {
+oemsg(int fd,const char *s,...) {
   va_list args;
   ufixnum n=0;
   void *v=NULL;
@@ -481,9 +483,12 @@ emsg(const char *s,...) {
   va_start(args,s);
   vsnprintf(v,n,s,args);
   va_end(args);
-  return write(2,v,n-1) ? n : -1;
+  return write(fd,v,n-1) ? n : -1;
 }
 
+#define omsg(a_...) oemsg(1,a_)
+#define emsg(a_...) oemsg(2,a_)
+
 EXTER char *heap_end;			/*  heap end  */
 EXTER char *core_end;			/*  core end  */
 EXTER 
@@ -536,7 +541,6 @@ EXTER char *alloca_val;
 #define ON_STACK_CONS(x,y) (ALLOCA_CONS_ALIGN(1), on_stack_cons(x,y)) 
 
 
-object make_si_sfun();
 
 /*FIXME -- this is an effort to minimize uninitialized garbage in the
   stack.  THe only comprehensive solution appears to be to wipe the
@@ -605,7 +609,7 @@ object make_si_sfun();
 #define RETURN2I(x,y)   RETURN(2,fixnum,x,(RV(y)))
 /* #define RETURN1(x) RETURN(1,object,x,) */
 #define RETURN1(x) return(x)
-#define RETURN0 do {vs_top=(object *)vals-1;return Cnil;} while (0)
+#define RETURN0 do {vs_top=vals ? (object *)vals-1 : base;return Cnil;} while (0)
 
 #define RV(x) ({if (_p) *_p++ = x;})
 
@@ -621,8 +625,6 @@ object make_si_sfun();
 
 
 
-object funcall_cfun(void(*)(),int,...);
-object clear_compiler_properties();
 EXTER object sSlambda_block_expanded;
 
 # ifdef __GNUC__ 
@@ -653,7 +655,6 @@ EXTER object sSlambda_block_expanded;
     do{ if(signals_allowed ==0) /* should not get here*/gcl_abort(); \
    raise_pending_signals(sig_safe)}while(0)
 
-void raise_pending_signals();
 
 EXTER unsigned plong signals_allowed, signals_pending;
 
