@@ -1,12 +1,5 @@
 #include "linux.h"
 
-#ifdef IN_GBC
-#undef MPROTECT_ACTION_FLAGS
-#define MPROTECT_ACTION_FLAGS SA_RESTART|SA_SIGINFO
-#define GET_FAULT_ADDR(sig,code,sv,a) \
- ((siginfo_t *)code)->si_addr
-#endif
-
 #define SGC
 
 #define CLEAR_CACHE_LINE_SIZE 32
