@@ -87,7 +87,8 @@ struct key {short n,allow_other_keys;
 /* cfun.c:283:OF */ extern object make_macro_internal (char *s, void(*f)()); /* (s, f) char *s; int (*f)(); */
 /* cfun.c:299:OF */ extern object make_si_sfun_internal (char *s, object (*f)(), int argd); /* (s, f, argd) char *s; int (*f)(); int argd; */
 /* cfun.c:322:OF */ extern object make_si_function_internal (char *s, void (*f) ()); /* (s, f) char *s; int (*f)(); */
-/* cfun.c:341:OF */ extern object make_special_form_internal (char *s, void * f); /* (s, f) char *s; int (*f)(); */
+/* cfun.c:341:OF */ extern object make_special_form_internal (char *s, void *f); /* (s, f) char *s; int (*f)(); */
+/* cfun.c:341:OF */ extern object make_si_special_form_internal (char *s, void *f); /* (s, f) char *s; int (*f)(); */
 /* cfun.c:371:OF */ extern void turbo_closure (object fun); /* (fun) object fun; */
 /* cfun.c:403:OF */ extern void gcl_init_cfun (void); /* () */
 /* cmac.c:191:OF */ extern void gcl_init_cmac (void); /* () */
@@ -1431,7 +1432,7 @@ object macro_def_int(object);
 
 int reset_plt(void);
 
-int msystem(const char *);
+int msystem(char *);
 
 fcomplex object_to_fcomplex(object);
 
@@ -1449,7 +1450,7 @@ void init_shared_memory(void);
 
 void * object_to_pointer(object);
 
-void * alloca(size_t);
+void * alloca(unsigned long);
 
 object make_dcomplex(dcomplex);
 
@@ -1603,3 +1604,75 @@ fresh_synonym_stream_to_terminal_io(void);
 
 void
 set_array_elttype(object,enum aelttype);
+
+object
+apply_format_function(object,object,object,object,object,object);
+
+object
+fSstring_match2(object,object);
+
+object
+aelttype_list(void);
+
+object alloc_simple_string(fixnum);
+object alloc_string(fixnum);
+object append(object,object);
+object car(object);
+object cdr(object);
+object copy_list(object);
+object copy_simple_string(object);
+object current_package(void);
+object find_package(object);
+object find_symbol(object,object);
+object getf(object,object,object);
+object intern(object,object);
+object make_constant(char *,object);
+object make_keyword(char *);
+object make_ordinary(char *);
+object make_si_constant(char *,object);
+object make_si_ordinary(char *);
+object make_si_special(char *,object);
+object make_special(char *,object);
+object nreverse(object);
+double object_to_double(object);
+object open_stream(object,enum smmode,object,object);
+object putf(object,object,object);
+object putprop(object,object,object);
+object read_object(object);
+object read_object_non_recursive(object);
+object make_symbol(object);
+object reverse(object);
+object alloc_bitvector(fixnum);
+object alloc_simple_bitvector(fixnum);
+object alloc_simple_vector(fixnum);
+object alloc_vector(fixnum,enum aelttype);
+object coerce_to_character(object);
+object peek_char(bool,object);
+object prin1(object,object);
+object read_char(object);
+object make_string_output_stream(int);
+object make_gmp_ordinary(char *);
+void *malloc(size_t);
+void *realloc(void *,size_t);
+void *alloc_contblock(size_t);
+void *alloc_relblock(size_t);
+object ifuncall(object,int,...);
+object list(fixnum,...);
+object listA(fixnum,...);
+object vs_overflow(void);
+object make_fixnum1(long);
+object make_shortfloat(float);
+long fixint(object);
+object read_char1(object,object);
+object Iapply_fun_n(object,int,int,...);
+object Iapply_fun_n2(object,int,int,...);
+object Ifuncall_n(object,int,...);
+object funcall_cfun(void(*)(),int,...);
+int gcl_init_cmp_anon(void);
+int is_bigger_fixnum(void *);
+int is_text_addr(void *);
+int seek_to_end_ofile(FILE *);
+void stack_list(void);
+void *msbrk(intptr_t);
+int msbrk_init(void);
+int msbrk_end(void);
