@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -122,8 +123,12 @@ expand_contblock_array(void) {
 static void
 contblock_array_push(void *p) {
 
+  ufixnum f=contblock_array==Cnil ? 0 : contblock_array->v.v_fillp;/*FIXME*/
+
   expand_contblock_array();
-  contblock_array->v.v_self[contblock_array->v.v_fillp]=p;
+  memmove(contblock_array->v.v_self+f+1,contblock_array->v.v_self+f,
+	  (contblock_array->v.v_fillp-f)*sizeof(*contblock_array->v.v_self));
+  contblock_array->v.v_self[f]=p;
   contblock_array->v.v_fillp++;
 
 }
@@ -222,8 +227,11 @@ add_page_to_freelist(char *p, struct typemanager *tm) {
      are many arrays in a system to make the headers not writable,
      but just SGC_TOUCH the header each time you write to it.   this
      is what is done with t_structure */
-  if (t==(tm_of(t_array)->tm_type))
-    pp->sgc_flags|=SGC_PERM_WRITABLE;
+  /* if (t==(tm_of(t_array)->tm_type)) */
+  /*   pp->sgc_flags|=SGC_PERM_WRITABLE; */
+  /* The SGC_PERM_WRITABLE facility is no longer used in favor of
+     SGC_TOUCH.  Implicitly grouping object types by size is
+     unreliable.*/
 
 #endif 
 
@@ -314,20 +322,6 @@ int reserve_pages_for_signal_handler=30;
    reserve_pages_for_signal_handler pages on hand in the hole
  */
 
-void
-empty_relblock(void) {
-
-  object o=sSAleaf_collection_thresholdA->s.s_dbind;
-  
-  sSAleaf_collection_thresholdA->s.s_dbind=make_fixnum(0);
-  for (;!rb_emptyp();) {
-    tm_table[t_relocatable].tm_adjgbccnt--;
-    GBC(t_relocatable);
-  }
-  sSAleaf_collection_thresholdA->s.s_dbind=o;
-
-}
-
 void
 setup_rb(bool preserve_rb_pointerp) {
 
@@ -704,6 +698,21 @@ reset_contblock_freelist(void) {
   
 }
 
+void
+empty_relblock(void) {
+
+  object o=sSAleaf_collection_thresholdA->s.s_dbind;
+
+  sSAleaf_collection_thresholdA->s.s_dbind=make_fixnum(0);
+  for (;!rb_emptyp();) {
+    tm_table[t_relocatable].tm_adjgbccnt--;
+    expand_contblock_index_space();
+    GBC(t_relocatable);
+  }
+  sSAleaf_collection_thresholdA->s.s_dbind=o;
+
+}
+
 static inline void *
 alloc_from_freelist(struct typemanager *tm,fixnum n) {
 
@@ -790,6 +799,13 @@ too_full_p(struct typemanager *tm) {
 
 }
 
+DEFUN("POOL-STAT",object,fSpool_stat,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
+  pool_stat();
+  RETURN1(MMcons(make_fixnum(pool_pid),MMcons(make_fixnum(pool_n),MMcons(make_fixnum(pool_s),Cnil))));
+
+}
+
 static inline bool
 do_gc_p(struct typemanager *tm,fixnum n) {
 
@@ -995,6 +1011,11 @@ alloc_contblock_no_gc(size_t n,char *limit) {
   void *p;
   
   n=CEI(n,CPTR_SIZE);
+
+  /*This is called from GBC so we do not want to expand the contblock index*/
+  if (cbv->v.v_fillp+1==cbv->v.v_dim ||
+      contblock_array->v.v_fillp==contblock_array->v.v_dim)
+    return NULL;
   
   if ((p=alloc_from_freelist(tm,n)))
     return p;
@@ -1182,7 +1203,7 @@ init_tm(enum type t, char *name, int elsize, int nelts, int sgc,int distinct) {
    homogenized and integrated into the traditional unix sequence for
    simplicity.  set_maxpage is overloaded, and the positioning of its
    call is too fragile.  20050115 CM*/
-static int gcl_alloc_initialized;
+int gcl_alloc_initialized;
 
 object malloc_list=Cnil;
 
@@ -1199,6 +1220,15 @@ gcl_init_alloc(void *cs_start) {
 
   fixnum cssize=(1L<<23);
 
+#ifdef GCL_GPROF
+  if (raw_image) {
+    sigset_t prof;
+    sigemptyset(&prof);
+    sigaddset(&prof,SIGPROF);
+    sigprocmask(SIG_BLOCK,&prof,NULL);
+  }
+#endif
+
   prelink_init();
   
 #ifdef RECREATE_HEAP
@@ -1272,8 +1302,14 @@ gcl_init_alloc(void *cs_start) {
 
 #endif
 
+#ifdef INITIALIZE_BRK
+  INITIALIZE_BRK;
+#endif
+
   update_real_maxpage();
 
+  cumulative_allocation=recent_allocation=0;
+
   if (gcl_alloc_initialized) {
     maybe_set_hole_from_maxpages();
     return;
@@ -1283,7 +1319,7 @@ gcl_init_alloc(void *cs_start) {
   INIT_ALLOC;
 #endif  
 
-  initial_sbrk=data_start=heap_end;
+  data_start=heap_end;
   first_data_page=page(data_start);
   
   /* Unused (at present) tm_distinct flag added.  Note that if cons
@@ -1639,10 +1675,6 @@ bool writable_malloc=0;
 static void *
 malloc_internal(size_t size) {
 
-#ifdef CAN_UNRANDOMIZE_SBRK
-  if (core_end && core_end!=sbrk(0))/*malloc before main in saved_image*/
-    return sbrk(size);/*will never get to gcl_init_alloc, so brk point irrelevant*/
-#endif
   if (!gcl_alloc_initialized) {
     static bool recursive_malloc;
     if (recursive_malloc)
