@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994  W. Schelter
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -430,7 +431,9 @@ fSmake_vector(object etp,fixnum n,object adjp,object fp,object displaced_to,fixn
 }
 #endif
 
-DEFUN("AELTTYPE-LIST",object,fSaelttype_list,SI,0,0,NONE,OO,OO,OO,OO,(),"") {
+
+object
+aelttype_list(void) {
 
   aet_type_struct *p,*pe;
   object f=Cnil,x,y=OBJNULL;
@@ -443,6 +446,7 @@ DEFUN("AELTTYPE-LIST",object,fSaelttype_list,SI,0,0,NONE,OO,OO,OO,OO,(),"") {
   return f;
 
 }
+DEFCONST("+ARRAY-TYPES+",sSParray_typesP,SI,aelttype_list(),"");
   
 
 DEFUN("GET-AELTTYPE",object,fSget_aelttype,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
@@ -621,7 +625,7 @@ set_displaced_body_ptr(object from_array) {
 
     enum aelttype typ =Iarray_element_type(from_array);
     object dest_array=displaced->c.c_car;
-    int offset=fix(displaced->c.c_cdr);
+    int offset=fix(Scdr(displaced));
 
     if (typ == aet_bit) {
       if (Iarray_element_type(dest_array)==aet_bit)
@@ -935,6 +939,7 @@ array_allocself(object x, int staticp, object dflt)
 		break;
 	case aet_bit:
 	  n=ceil(n,BV_ALLOC);
+	  n++;/*allow for arrays displaced to end BV_ALLOC access*/
 	  SET_BV_OFFSET(x,0);
 	case aet_fix:
 	case aet_nnfix:
