@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -159,6 +160,8 @@ DEFUN("FUNCTION-NAME",object,fSfunction_name,SI,1,1,NONE,OO,OO,OO,OO,(object x),
 
 DEFUN("FSET",object,fSfset,SI,2,2,NONE,OO,OO,OO,OO,(object sym,object function),"") {
 
+  object x;
+
   if (type_of(sym)!=t_symbol)
     sym=ifuncall1(sSfunid_to_sym,sym);
   
@@ -170,9 +173,6 @@ DEFUN("FSET",object,fSfset,SI,2,2,NONE,OO,OO,OO,OO,(object sym,object function),
       FEerror("~S, a special form, cannot be redefined.",
 	      1, sym);
   }
-  if (sym->s.s_hpack == lisp_package &&
-      sym->s.s_gfdef != OBJNULL && !raw_image && sLwarn->s.s_gfdef)
-    ifuncall2(sLwarn,make_simple_string("~S is being redefined."),sym);
   sym = clear_compiler_properties(sym,function);
   if (type_of(function) == t_function) {
     sym->s.s_gfdef = function;
@@ -189,9 +189,11 @@ DEFUN("FSET",object,fSfset,SI,2,2,NONE,OO,OO,OO,OO,(object sym,object function),
   }
   
   sym->s.s_sfdef=NOT_SPECIAL;/*FIXME?*/
-  if (function->fun.fun_plist!=Cnil)
+  if (function->fun.fun_plist!=Cnil) {
     function->fun.fun_plist->c.c_cdr->c.c_cdr->c.c_cdr->c.c_cdr->c.c_cdr->c.c_car=sym;/*FIXME*/
-  
+    x=function->fun.fun_plist->c.c_cdr->c.c_cdr->c.c_cdr->c.c_car;
+    function->fun.fun_plist->c.c_cdr->c.c_cdr->c.c_cdr->c.c_car=x==Cnil ? sLAload_truenameA->s.s_dbind : x;
+  }
   RETURN1(function);
 
 }
@@ -253,10 +255,6 @@ DEFUN("FMAKUNBOUND",object,fLfmakunbound,LISP,1,1,NONE,OO,OO,OO,OO,(object sym),
   }
   remf(&(rsym->s.s_plist),sStraced);
   clear_compiler_properties(rsym,Cnil);
-  if (rsym->s.s_hpack == lisp_package &&
-      rsym->s.s_gfdef != OBJNULL && !raw_image) {
-    ifuncall2(sLwarn, make_simple_string("~S is being redefined."), rsym);
-  }
 
   rsym->s.s_gfdef = OBJNULL;
   rsym->s.s_mflag = FALSE;
@@ -312,17 +310,9 @@ static object
 setf(object place, object form)
 {
 	object fun;
-	object *vs = vs_top;
-	void (*f)();
 	object args;
 	object x,result,y;
-	int i,nka=0;
-/*  	extern void siLaset(void); */
-/*  	extern void siLsvset(void); */
-	extern void siLelt_set();
-	extern void siLchar_set();
-/*  	extern void siLfill_pointer_set(void); */
-	extern void siLhash_set();
+	int i;
 
 	if (!consp(place)) {
 	  setq(place, result=Ieval1(form));
@@ -335,7 +325,7 @@ setf(object place, object form)
 	args = place->c.c_cdr;
 
 	{
-	  object p=find_package(make_simple_string("COMMON-LISP"));
+	  object p=lisp_package;
 	  char *s;
 
 	  if (fun->s.s_hpack==p && fun->s.s_name->st.st_self[0]=='C' && fun->s.s_name->st.st_self[VLEN(fun->s.s_name)-1]=='R' && VLEN(fun->s.s_name)!=3) {
@@ -370,9 +360,12 @@ setf(object place, object form)
 	  return Ieval1(MMcons(find_symbol(str("ASET"),system_package),MMcons(form,args)));
 	if (fun == sLsvref)
 	  return Ieval1(MMcons(find_symbol(str("SVSET"),system_package),append(args,MMcons(form,Cnil))));
-	if (fun == sLelt) { f = siLelt_set; goto EVAL; }
-	if (fun == sLchar) { f = siLchar_set; goto EVAL; }
-	if (fun == sLschar) { f = siLchar_set; goto EVAL; }
+	if (fun == sLelt)
+	  return Ieval1(MMcons(find_symbol(str("ELT-SET"),system_package),append(args,MMcons(form,Cnil))));
+	if (fun == sLchar)
+	  return Ieval1(MMcons(find_symbol(str("CHAR-SET"),system_package),append(args,MMcons(form,Cnil))));
+	if (fun == sLschar)
+	  return Ieval1(MMcons(find_symbol(str("SCHAR-SET"),system_package),append(args,MMcons(form,Cnil))));
 	if (fun == sLfill_pointer) 
 	  return Ieval1(MMcons(find_symbol(str("FILL-POINTER-SET"),system_package),append(args,MMcons(form,Cnil))));
 	if (fun == sLgethash) 
@@ -420,18 +413,6 @@ setf(object place, object form)
 	}
 	return result;
 
-EVAL:
-	for (;!endp(args);args=args->c.c_cdr)
-	  eval_push(args->c.c_car);
-	/* if (f!=siLaset) eval_push(form); */
-	if (nka && vs_top-vs>nka) {
-	  vs[nka]=vs_base[0];
-	  vs_top=vs+nka+1;
-	}
-	nka=0;
-	vs_base = vs;
-	(*f)();
-	return vs_base[0];
 
 OTHERWISE:
 	vs_base = vs_top;
