@@ -1,7 +1,9 @@
 #include <stdlib.h>
 void bcopy(const void *s1, void *s2, size_t n)
-{ while (n-- > 0) {
-    *((char *)s2)++ = *((char *)s1)++;
+{ const char *c1=s1;
+  char *c2=s2;
+  while (n-- > 0) {
+    *c2++ = *c1++;
 }
 }
 
