@@ -1,15 +1,16 @@
+/* Copyright (C) 2024 Camm Maguire */
 #include "include.h"
 
 
-DEFUN("TP0",fixnum,fStp0,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp0(x);}
-DEFUN("TP1",fixnum,fStp1,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp1(x);}
-DEFUN("TP2",fixnum,fStp2,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp2(x);}
-DEFUN("TP3",fixnum,fStp3,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp3(x);}
-DEFUN("TP4",fixnum,fStp4,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp4(x);}
-DEFUN("TP5",fixnum,fStp5,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp5(x);}
-DEFUN("TP6",fixnum,fStp6,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp6(x);}
-DEFUN("TP7",fixnum,fStp7,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp7(x);}
-DEFUN("TP8",fixnum,fStp8,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return tp8(x);}
+DEFUN("TP0",object,fStp0,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp0(x);}
+DEFUN("TP1",object,fStp1,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp1(x);}
+DEFUN("TP2",object,fStp2,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp2(x);}
+DEFUN("TP3",object,fStp3,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp3(x);}
+DEFUN("TP4",object,fStp4,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp4(x);}
+DEFUN("TP5",object,fStp5,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp5(x);}
+DEFUN("TP6",object,fStp6,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp6(x);}
+DEFUN("TP7",object,fStp7,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp7(x);}
+DEFUN("TP8",object,fStp8,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {return (object)(fixnum)tp8(x);}
 
 DEFUN("C-OBJECT-==",object,fSc_object_eq,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
   RETURN1(x==y?Ct:Cnil);
@@ -42,26 +43,26 @@ DEFUN("C-DCOMPLEX-==",object,fSc_dcomplex_eq,SI,2,2,NONE,OO,OO,OO,OO,(object x,o
   RETURN1(lfc(x)==lfc(y)?Ct:Cnil);
 }
 
-DEFUN("C+",fixnum,fScp,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x+y);
+DEFUN("C+",object,fScp,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x+y));
 }
-DEFUN("&",fixnum,fSand,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x&y);
+DEFUN("&",object,fSand,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x&y));
 }
-DEFUN("|",fixnum,fSor,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x|y);
+DEFUN("|",object,fSor,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x|y));
 }
-DEFUN("^",fixnum,fSxor,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x^y);
+DEFUN("^",object,fSxor,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x^y));
 }
-DEFUN("~",fixnum,fSnot,SI,1,1,NONE,II,OO,OO,OO,(fixnum x),"") {
-  RETURN1(~x);
+DEFUN("~",object,fSnot,SI,1,1,NONE,II,OO,OO,OO,(fixnum x),"") {
+  RETURN1((object)~x);
 }
-DEFUN("<<",fixnum,fSlshft,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x<<y);
+DEFUN("<<",object,fSlshft,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x<<y));
 }
-DEFUN(">>",fixnum,fSrshft,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
-  RETURN1(x>>y);
+DEFUN(">>",object,fSrshft,SI,2,2,NONE,II,IO,OO,OO,(fixnum x,fixnum y),"") {
+  RETURN1((object)(x>>y));
 }
 
 static inline bool
@@ -190,7 +191,7 @@ DEFUN("SUBSETP",object,fLsubsetp,LISP,2,2,NONE,OO,OO,OO,OO,(object x,object y),"
   if (!listp(y))/*FIXME checktype*/
     TYPE_ERROR(y,sLlist);
   for (;consp(x);x=x->c.c_cdr)
-    if (FFN(fLmember(x->c.c_car,y))==Cnil)
+    if (FFN(fLmember)(x->c.c_car,y)==Cnil)
       RETURN1(Cnil);
 
   RETURN1(Ct);
@@ -542,7 +543,7 @@ DEFUN("WILD-PATHNAME-P",object,fLwild_pathname_p,LISP,1,2,NONE,OO,OO,OO,OO,(obje
   return Cnil;
 }
 
-DEFUN("SET-DIFFERENCE-EQ",object,fSset_difference_eq,SI,2,8,NONE,OO,OO,OO,OO,
+DEFUN("SET-DIFFERENCE",object,fLset_difference,LISP,2,8,NONE,OO,OO,OO,OO,
 	  (object x,object y,...),"") {
   object z=Cnil,yy;
   for (;x!=Cnil;x=x->c.c_cdr) {
@@ -554,7 +555,7 @@ DEFUN("SET-DIFFERENCE-EQ",object,fSset_difference_eq,SI,2,8,NONE,OO,OO,OO,OO,
 
 }
 
-DEFUN("UNION-EQ",object,fSunion_eq,SI,2,8,NONE,OO,OO,OO,OO,
+DEFUN("UNION",object,fLunion,LISP,2,8,NONE,OO,OO,OO,OO,
 	  (object x,object y,...),"") {
   object z=y,yy;
   for (;x!=Cnil;x=x->c.c_cdr) {
@@ -566,7 +567,7 @@ DEFUN("UNION-EQ",object,fSunion_eq,SI,2,8,NONE,OO,OO,OO,OO,
 
 }
 
-DEFUN("NUNION-EQ",object,fSnunion_eq,SI,2,8,NONE,OO,OO,OO,OO,
+DEFUN("NUNION",object,fLnunion,LISP,2,8,NONE,OO,OO,OO,OO,
 	  (object x,object y,...),"") {
   object z=Cnil,zp=z,yy;
   for (;x!=Cnil;x=x->c.c_cdr) {
@@ -581,7 +582,7 @@ DEFUN("NUNION-EQ",object,fSnunion_eq,SI,2,8,NONE,OO,OO,OO,OO,
 
 }
 
-DEFUN("INTERSECTION-EQ",object,fSintersection_eq,SI,2,8,NONE,OO,OO,OO,OO,
+DEFUN("INTERSECTION",object,fLintersection,LISP,2,8,NONE,OO,OO,OO,OO,
 	  (object x,object y,...),"") {
   object z=Cnil,yy;
   for (;x!=Cnil;x=x->c.c_cdr) {
@@ -626,6 +627,75 @@ DEFUN("HASH-SET",object,fShash_set,SI,3,3,NONE,OO,OO,OO,OO,(object x,object y,ob
 
 }
 
+DEFUN("COMPLEX",object,fLcomplex,LISP,1,2,NONE,OO,OO,OO,OO,(object r,...),"") {
+  fixnum nargs=INIT_NARGS(1);
+  object l=Cnil,f=OBJNULL,i;
+  va_list ap;
+
+  va_start(ap,r);
+  i=NEXT_ARG(nargs,ap,l,f,make_fixnum(0));
+  va_end(ap);
+
+  check_type_or_rational_float(&r);
+  check_type_or_rational_float(&i);
+
+  RETURN1(make_complex(r,i));
+
+}
+
+DEFUN("FLOAT",object,fLfloat,LISP,1,2,NONE,OO,OO,OO,OO,(object x,...),"") {
+
+  fixnum nargs=INIT_NARGS(1);
+  object l=Cnil,f=OBJNULL,y;
+  va_list ap;
+  double d;
+  enum type t;
+
+  va_start(ap,x);
+  y=NEXT_ARG(nargs,ap,l,f,(t=type_of(x))==t_shortfloat || t==t_longfloat ? x : make_longfloat(0.0));
+  va_end(ap);
+
+  /* check_type_float(&x); */
+  check_type_float(&y);
+
+  t=type_of(y);
+
+  switch (type_of(x)) {
+  case t_fixnum:
+    if (t == t_shortfloat)
+      x = make_shortfloat((shortfloat)(fix(x)));
+    else
+      x = make_longfloat((double)(fix(x)));
+    break;
+
+  case t_bignum:
+  case t_ratio:
+    d = number_to_double(x);
+    if (t == t_shortfloat)
+      x = make_shortfloat((shortfloat)d);
+    else
+      x = make_longfloat(d);
+    break;
+
+  case t_shortfloat:
+    if (t == t_longfloat)
+      x = make_longfloat((double)(sf(x)));
+    break;
+
+  case t_longfloat:
+    if (t == t_shortfloat)
+      x = make_shortfloat((shortfloat)(lf(x)));
+    break;
+
+  default:
+    FEwrong_type_argument(TSor_rational_float, x);
+  }
+
+  RETURN1(x);
+
+}
+
+
 #ifndef NO_BOOT_H
 #include "boot.h"
 #endif
