@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -34,7 +35,7 @@ object sSterminal_interrupt;
 void
 assert_error(const char *a,unsigned l,const char *f,const char *n) {
 
-  if (!raw_image && core_end && core_end==sbrk(0))
+  if (!raw_image && core_end && core_end==sbrk(0) && errno!=ENOMEM && ihs_top>=ihs_org)
     FEerror("The assertion ~a on line ~a of ~a in function ~a failed: ~a",5,
 	    make_simple_string(a),make_fixnum(l),
 	    make_simple_string(f),make_simple_string(n),make_simple_string(strerror(errno)));
@@ -341,6 +342,7 @@ DEFUN("SCH-FRS-BASE",object,fSsch_frs_base,SI,2,2,NONE,OO,OO,OO,OO,(object x0,ob
 DEFUNM("INTERNAL-SUPER-GO",object,fSinternal_super_go,SI,3,3,NONE,OO,OO,OO,OO,(object tag,object x1,object x2),"") {
   frame_ptr fr;
   fixnum vals=(fixnum)fcall.valp;
+  object *base=vs_top;
   
   fr = frs_sch(tag);
   if (fr == NULL)
@@ -398,9 +400,9 @@ void
 check_arg_failed(int n)
 {
   if (n<vs_top-vs_base)
-    FEtoo_many_arguments(vs_base,vs_top);
-  else
     FEtoo_few_arguments(vs_base,vs_top);
+  else
+    FEtoo_many_arguments(vs_base,vs_top);
 }
 
 void
