@@ -1,5 +1,6 @@
 /*
   Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+  Copyright (C) 2024 Camm Maguire
   
   This file is part of GNU Common Lisp, herein referred to as GCL
   
@@ -303,7 +304,6 @@ enter_mark_origin(object *p) {
 
 /* Whenever two arrays are linked together by displacement,
    if one is live, the other will be made live */
-#define mark_displaced_field(ar) mark_object(ar->a.a_displaced)
 
 #define LINK_ARRAY_MARKED(x_) ((*(unsigned long *)(x_))&0x1)
 #define MARK_LINK_ARRAY(x_) ((*(unsigned long *)(x_))|=1UL)
@@ -478,7 +478,7 @@ mark_object_array(object *o,object *oe) {
 static void
 mark_object1(object x) {
 
-  fixnum i,j=0;/*FIXME*/
+  fixnum i,j;
 
   if (is_marked_or_free(x))
     return;
@@ -555,43 +555,20 @@ mark_object1(object x) {
 
   case t_simple_vector:
   case t_simple_bitvector:
+  case t_simple_string:
   case t_vector:
   case t_bitvector:
+  case t_string:
 
-    switch(j ? j : (enum aelttype)x->v.v_elttype) {
+    if (x->v.v_elttype==aet_object && ADISP(x)->c.c_car==Cnil)
+      mark_object_array(x->v.v_self,x->v.v_self+x->v.v_dim);
 
-    case aet_lf:
-      j= sizeof(longfloat)*x->v.v_dim;
-      if ((COLLECT_RELBLOCK_P) &&  (void *)x->v.v_self>=(void *)heap_end)
-	rb_pointer=PCEI(rb_pointer,sizeof(double));		/*FIXME GC space violation*/
-      break;
+    j=x->v.v_eltsize ? (1<<(x->v.v_eltsize-1)) : x->v.v_eltsize;
 
-    case aet_bit:
-      j=ceil(BV_OFFSET(x)+x->bv.bv_dim,BV_ALLOC)*sizeof(*x->bv.bv_self);
-      break;
+    if ((COLLECT_RELBLOCK_P) && (void *)x->v.v_self>=(void *)heap_end && j>sizeof(long))
+      rb_pointer=PCEI(rb_pointer,j); /*FIXME GC space violation*/
 
-    case aet_char:
-    case aet_uchar:
-      j=sizeof(char)*x->v.v_dim;
-      break;
-
-    case aet_short:
-    case aet_ushort:
-      j=sizeof(short)*x->v.v_dim;
-      break;
-
-    case aet_object:
-      if (ADISP(x)->c.c_car==Cnil)
-	mark_object_array(x->v.v_self,x->v.v_self+x->v.v_dim);
-
-    default:
-      j=sizeof(fixnum)*x->v.v_dim;
-
-    }
-
-  case t_simple_string:
-  case t_string:/*FIXME*/
-    j=j ? j : x->st.st_dim;
+    j=j ? j*x->v.v_dim : ceil(BV_OFFSET(x)+x->bv.bv_dim,BV_ALLOC)*sizeof(*x->bv.bv_self);
 
     if (ADISP(x)->c.c_car==Cnil)
       MARK_LEAF_DATA(x,x->v.v_self,j);
@@ -689,7 +666,7 @@ mark_object1(object x) {
     mark_object(x->pn.pn_namestring);
     break;
     
-  case t_function:	
+  case t_function:
     mark_object(x->fun.fun_data);
     mark_object(x->fun.fun_plist);
     if (x->fun.fun_env != def_env && x->fun.fun_env != src_env) {
@@ -1068,6 +1045,7 @@ GBC(enum type t) {
   }
 
   ngc_thresh=fix(sSAleaf_collection_thresholdA->s.s_dbind);
+  cumulative_allocation+=recent_allocation;
   recent_allocation=0;
 
   if (in_signal_handler && t == t_relocatable)
@@ -1121,12 +1099,12 @@ GBC(enum type t) {
       gc_time=0;
 
 #ifdef SGC
-    emsg("[%s for %ld %s pages..",
+    omsg("[%s for %ld %s pages..",
 	 (sgc_enabled ? "SGC" : "GC"),
 	 (sgc_enabled ? sgc_count_type(t) : tm_of(t)->tm_npage),
 	 (tm_table[(int)t].tm_name)+1);
 #else
-    emsg("[%s for %ld %s pages..",
+    omsg("[%s for %ld %s pages..",
 	 ("GC"),
 	 (tm_of(t)->tm_npage),
 	 (tm_table[(int)t].tm_name)+1);
@@ -1252,9 +1230,9 @@ GBC(enum type t) {
   if (sSAnotify_gbcA->s.s_dbind != Cnil) {
     
     if (gc_recursive)
-      emsg("(T=...).GC finished]\n");
+      omsg("(T=...).GC finished]\n");
     else
-      emsg("(T=%d).GC finished]\n",gc_start);
+      omsg("(T=%d).GC finished]\n",gc_start);
 
   }
   
@@ -1270,16 +1248,16 @@ static void
 FFN(siLheap_report)(void) {
 
   int i;
+  extern void *shared_lib_start;
   
   check_arg(0);
   
   vs_check_push(make_fixnum(sizeof(fixnum)*CHAR_SIZE));
   vs_push(make_fixnum(PAGESIZE));
   vs_push(make_fixnum((ufixnum)data_start));
-  vs_push(make_fixnum((ufixnum)data_start+(real_maxpage<<PAGEWIDTH)));
-  vs_push(make_fixnum(0));/*SHARED_LIB_HEAP_CEILING*/
-  i=sizeof(fixnum)*CHAR_SIZE-2;
-  i=1<<i;
+  vs_push(make_fixnum(((unsigned long)real_maxpage)<<PAGEWIDTH));
+  vs_push(make_fixnum((ufixnum)shared_lib_start));
+  i=getpagesize();
   vs_push(make_fixnum(((unsigned long)cs_base+i-1)&-i));
   vs_push(make_fixnum(labs(cs_base-cs_org)));
   vs_push(make_fixnum((CSTACK_DIRECTION+1)>>1));
@@ -1300,6 +1278,13 @@ FFN(siLheap_report)(void) {
 #endif
 
 }  
+DEFUN("CUMULATIVE-ALLOCATION",object,fScumulative_allocation,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+  ufixnum x=(cumulative_allocation+recent_allocation)>>20;
+  if ((fixnum)x>=0)
+    RETURN1(make_fixnum(x));
+  mpz_set_ui(MP(big_fixnum1),x);
+  RETURN1(replace_big(big_fixnum1));
+}
 
 DEFUN("ROOM-REPORT",object,fSroom_report,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
 
