@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -154,7 +155,8 @@ clear_c_stack(VOL unsigned n) {
 
 }
 
-fixnum log_maxpage_bound=sizeof(fixnum)*8-1;
+static fixnum log_maxpage_bound=sizeof(fixnum)*8-1;
+static fixnum mem_bound=sizeof(fixnum)*8-1;
 
 int
 mbrk(void *v) {
@@ -162,10 +164,6 @@ mbrk(void *v) {
   ufixnum uv=(ufixnum)v,uc=(ufixnum)sbrk(0),ux,um;
   fixnum m=((1UL<<(sizeof(fixnum)*8-1))-1);
 
-#ifdef MAX_BRK /*GNU Hurd fragmentation bug*/
-  if ((ufixnum)v>MAX_BRK) return -1;
-#endif
-
   if (uv<uc) {
     um=uv;
     ux=uc;
@@ -181,12 +179,73 @@ mbrk(void *v) {
 
 }
     
+static char *
+next_line(int l,ufixnum *s) {
+
+  size_t n;
+  ssize_t i;
+  char *p;
+
+  if (*s && (n=strlen(FN1))) {
+    memmove(FN1,FN1+n+1,sizeof(FN1)-(n+1));
+    *s-=n+1;
+  }
+
+  massert(*s<sizeof(FN1));
+  massert((i=read(l,FN1+*s,sizeof(FN1)-*s))>=0);
+  *s+=i;
+
+  if (!(p=memchr(FN1,'\n',*s)))
+    return NULL;
+
+  *p=0;
+  return FN1;
+
+}
+
+static void
+parse_meminfo_decimal(char *c,char *h,ufixnum *t) {
+
+  ufixnum n=strlen(h);
+
+  if (memcmp(h,c,n))
+    return;
+
+  massert(sscanf(c+n,"%ld kB",t)==1);
+  *t<<=10;
+  *t>>=PAGEWIDTH;
+
+}
+
+static void
+parse_proc_meminfo(ufixnum *t,ufixnum *f,ufixnum *st,ufixnum *sf) {
+
+  int l;
+  ufixnum r=0;
+  char *s="/proc/meminfo",*c;
+
+  *t=*f=*st=*sf=0;
+
+  massert((l=open(s,O_RDONLY))>=0);
+
+  for (;(c=next_line(l,&r));) {
+    parse_meminfo_decimal(c,"MemTotal:",t);
+    parse_meminfo_decimal(c,"MemFree:",f);
+    parse_meminfo_decimal(c,"SwapTotal:",st);
+    parse_meminfo_decimal(c,"SwapFree:",sf);
+  }
+
+  massert(!close(l));
+
+}
+
+
 #if defined(__CYGWIN__)||defined(__MINGW32__)
 
 #include <windows.h>
 
 static ufixnum
-get_phys_pages_no_malloc(char n) {
+get_phys_pages_no_malloc(char n,char ramp) {
 
   MEMORYSTATUS m;
 
@@ -201,7 +260,7 @@ get_phys_pages_no_malloc(char n) {
 #include <sys/sysctl.h>
 
 static ufixnum
-get_phys_pages_no_malloc(char n) {
+get_phys_pages_no_malloc(char n,char ramp) {
 
   uint64_t s;
   size_t z=sizeof(s);
@@ -214,10 +273,10 @@ get_phys_pages_no_malloc(char n) {
 
 }
 
-#elif defined(__sun__) || defined(__GNU__)
+#elif defined(__sun__)
 
 static ufixnum
-get_phys_pages_no_malloc(char n) {
+get_phys_pages_no_malloc(char n,char ramp) {
 
   return sysconf(_SC_PHYS_PAGES);
 
@@ -229,7 +288,7 @@ get_phys_pages_no_malloc(char n) {
 #include <sys/sysctl.h>
 
 static ufixnum
-get_phys_pages_no_malloc(char n) {
+get_phys_pages_no_malloc(char n,char ramp) {
 
   size_t i,len=sizeof(i);
 
@@ -239,26 +298,41 @@ get_phys_pages_no_malloc(char n) {
 
 #else /*Linux*/
 
-#include <sys/sysinfo.h>
-
 static ufixnum
-get_phys_pages_no_malloc(char freep) {
+get_phys_pages_no_malloc(char freep,char ramp) {
 
-  struct sysinfo s;
+  ufixnum t,f,st,sf;
 
-  return sysinfo(&s) ? 0 : ((freep ? s.freeram : s.totalram)>>PAGEWIDTH)*s.mem_unit;
+  parse_proc_meminfo(&t,&f,&st,&sf);
+
+  return (freep ? (ramp ? f : f+sf) : (ramp ? t : t+st));
 
 }
 
+
 #endif
 
 static ufixnum
-get_phys_pages1(char freep) {
+get_phys_pages1(char freep,char ramp) {
 
-  return get_phys_pages_no_malloc(freep);
+  ufixnum p=get_phys_pages_no_malloc(freep,ramp);
+#if defined(__GNU__)/*FIXME -- no mmap overcommit as of yet*/
+  p>>=1;
+#endif
+  return p;
 
 }
 
+static int
+acceptable_log_maxpage_bound(ufixnum l) {
+
+  ufixnum max=(sizeof(fixnum)<<3)-1;
+
+  return l>max ? 0 :
+    (l==max ? 1 :
+     (void *)heap_end-data_start+CEI(rb_pointer-rb_begin(),PAGESIZE) < (1UL<<(l+1)));
+}
+
 static void
 get_gc_environ(void) {
 
@@ -270,6 +344,16 @@ get_gc_environ(void) {
     massert(mem_multiple>=0.0);
   }
 
+  mem_bound=sizeof(fixnum)*8-1;
+  if ((e=getenv("GCL_MEM_BOUND"))) {
+    ufixnum f;
+    massert(sscanf(e,"%lud",&f)==1);
+    if (acceptable_log_maxpage_bound(f)) {
+      mem_bound=f;
+      mem_multiple=1.0;
+    }
+  }
+
   gc_alloc_min=0.05;
   if ((e=getenv("GCL_GC_ALLOC_MIN"))) {
     massert(sscanf(e,"%lf",&gc_alloc_min)==1);
@@ -288,8 +372,10 @@ get_gc_environ(void) {
     massert(gc_page_max>=0.0);
   }
 
-  multiprocess_memory_pool=
-    (e=getenv("GCL_MULTIPROCESS_MEMORY_POOL")) && (*e=='t' || *e=='T');
+  multiprocess_memory_pool=getenv("GCL_MULTIPROCESS_MEMORY_POOL");
+  if (multiprocess_memory_pool &&
+      (*multiprocess_memory_pool=='t' || *multiprocess_memory_pool=='T'))/*GCL 2.6 compatability*/
+    multiprocess_memory_pool=getenv("HOME");
 
   wait_on_abort=0;
   if ((e=getenv("GCL_WAIT_ON_ABORT")))
@@ -318,17 +404,82 @@ setup_maxpages(double scale) {
   resv_pages=available_pages/100;
   available_pages-=resv_pages;
 
-  recent_allocation=0;
-
 }
 
-void *initial_sbrk=NULL;
+
+static void *
+next_shared_lib_map_no_malloc(void)  {
+
+#if !defined(DARWIN) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)/*FIXME*/
+
+  char *c,rwx[4];
+  ufixnum a,e,s=(ufixnum)sbrk(0),r;
+  int l;
+
+  massert((l=open("/proc/self/maps",O_RDONLY))!=-1);
+
+  for (a=r=0;(a<=s || !memcmp(rwx,"---",3)) && (c=next_line(l,&r));)
+    sscanf(c,"%lx-%lx %3s",&a,&e,rwx);
+
+  massert(!close(l));
+
+  return (void *)(c ? a : -1);
+
+#else
+
+  return (void *)-1;
+
+#endif
+}
+
+static void *stack_map_base=(void *)-1;
+void *shared_lib_start=(void *)-1;
+
+static int
+set_real_maxpage(void *beg) {
+
+  void *end,*cp;
+  ufixnum mp,sz;
+
+  end=(void *)ROUNDDN((void *)-1,PAGESIZE);
+  mp=page(end-beg);
+
+  mp=ufmin(mp,get_phys_pages1(0,0));
+
+  sz=ufmin(mem_bound,log_maxpage_bound);
+  sz=(1UL<<sz)+((1UL<<sz)-1);
+  mp=ufmin(mp,page(sz));
+
+#if defined(LOW_IM_FIX)
+  cp=(void *)(ufixnum)LOW_IM_FIX;
+#elif defined(IM_FIX_BASE)
+  cp=(void *)IM_FIX_BASE;
+#else
+  cp=(void *)-1;
+#endif
+  cp=cp<beg ? (void *)-1 : cp;
+  mp=ufmin(mp,page(cp-beg));
+
+  cp=alloca(1);
+  cp=cp<stack_map_base ? cp : stack_map_base;
+  cp=cp<beg ? (void *)-1 : cp;
+  mp=ufmin(mp,page(cp-beg));
+
+  cp=shared_lib_start=next_shared_lib_map_no_malloc();
+  cp=cp<beg ? (void *)-1 : cp;
+  mp=ufmin(mp,page(cp-beg));
+
+  real_maxpage=mp+page(beg);
+
+  return 0;
+
+}
 
 int
 update_real_maxpage(void) {
 
-  ufixnum i,j;
-  void *end,*cur,*beg;
+  void *beg;
+
 #ifdef __MINGW32__
   static fixnum n;
 
@@ -338,23 +489,13 @@ update_real_maxpage(void) {
   }
 #endif
 
-#ifdef DEFINED_REAL_MAXPAGE
-  real_maxpage=DEFINED_REAL_MAXPAGE;
-#else
-  massert(cur=sbrk(0));
-  beg=data_start ? data_start : cur;
-  for (i=0,j=(1L<<log_maxpage_bound);j>PAGESIZE;j>>=1)
-    if ((end=beg+i+j-PAGESIZE)>cur)
-      if (!mbrk(end)) {
-	real_maxpage=page(end);
-	i+=j;
-      }
-  massert(!mbrk(cur));
-#endif
-
-  phys_pages=ufmin(get_phys_pages1(0)+page(beg),real_maxpage)-page(beg);
-
   get_gc_environ();
+
+  massert(beg=data_start ? data_start : sbrk(0));
+  set_real_maxpage(beg);
+
+  phys_pages=ufmin(get_phys_pages1(0,1)+page(beg),real_maxpage)-page(beg);
+
   setup_maxpages(mem_multiple);
 
   return 0;
@@ -370,8 +511,6 @@ minimize_image(void) {
   nrbpage=0;
   resize_hole(0,t_relocatable,0);
 
-  gprof_cleanup();
-  
 #if defined(BSD) || defined(ATT)  
   mbrk(core_end=heap_end);
 #endif
@@ -387,16 +526,12 @@ minimize_image(void) {
 
 DEFUN("SET-LOG-MAXPAGE-BOUND",object,fSset_log_maxpage_bound,SI,1,1,NONE,II,OO,OO,OO,(fixnum l),"") {
 
-  void *end,*dend;
-  fixnum def=sizeof(fixnum)*8-1;
-
-  l=l<def ? l : def;
-  end=data_start+(1L<<l)-PAGESIZE;
   GBC(t_relocatable);
-  dend=heap_end+PAGESIZE+CEI(rb_pointer-rb_begin(),PAGESIZE);
-  if (end >= dend) {
+  if (acceptable_log_maxpage_bound(l)) {
+    unsetenv("GCL_MEM_MULTIPLE");/*invoking this function overrides mem_multiple*/
+    unsetenv("GCL_MEM_BOUND");/*invoking this function overrides mem_multiple*/
     minimize_image();
-    log_maxpage_bound=l;/*FIXME maybe this should be under mem_multiple, not over*/
+    log_maxpage_bound=l;
     update_real_maxpage();
     maybe_set_hole_from_maxpages();
   }
@@ -415,21 +550,39 @@ dir_name_length(const char *s) {
   return m;
 }
 
+int initializing_boot=0;
+
 void
 init_boot(void) {
 
   char *sysd=getenv("GCL_SYSDIR"),*d=sysd ? sysd : kcl_self;
+#ifndef __CYGWIN__
   void *v,*q;
+#endif
   char *z,*s="libboot.so";
   size_t m=sysd ? strlen(sysd) : dir_name_length(kcl_self),n=m+strlen(s)+1;
+  object omp=sSAoptimize_maximum_pagesA->s.s_dbind;
 
+  sSAoptimize_maximum_pagesA->s.s_dbind=Cnil;
   z=alloca(n);
   snprintf(z,n,"%-*.*s%s",(int)m,(int)m,d,s);
+#ifndef __CYGWIN__
   if (!(v=dlopen(z,RTLD_LAZY|RTLD_GLOBAL)))
     printf("%s\n",dlerror());
   if (!(q=dlsym(v,"gcl_init_boot")))
     printf("%s\n",dlerror());
+#endif
+  initializing_boot=1;
+#ifdef __CYGWIN__
+  {
+    extern void gcl_init_boot(void);
+    gcl_init_boot();
+  }
+#else
   ((void (*)())q)();
+#endif
+  initializing_boot=0;
+  sSAoptimize_maximum_pagesA->s.s_dbind=omp;
 
 }
 
@@ -469,7 +622,9 @@ gcl_cleanup(int gc) {
   {extern void _cleanup(void);_cleanup();}
 #endif
 
+#ifdef USE_GPROF
   gprof_cleanup();
+#endif
 
   if (gc) {
 
@@ -481,7 +636,7 @@ gcl_cleanup(int gc) {
 
     raw_image=FALSE;
     cs_org=0;
-    initial_sbrk=core_end;
+    msbrk_end();
 
   }
 
@@ -555,17 +710,24 @@ which(const char *n,char *o) {
 
 #endif
 
+DEFUN("KCL-SELF",object,fSkcl_self,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
+  return make_simple_string(kcl_self);
+
+}
+
 int
 main(int argc, char **argv, char **envp) {
 
   GET_FULL_PATH_SELF(kcl_self);
   *argv=kcl_self;
 
-#ifdef CAN_UNRANDOMIZE_SBRK
-#include <stdio.h>
-#include <stdlib.h>
-#include "unrandomize.h"
-#endif
+  vs_top = vs_base = vs_org;
+  ihs_top = ihs_org-1;
+  bds_top = bds_org-1;
+  frs_top = frs_org-1;
+
+#include "cstack.h"
 
   gcl_init_alloc(alloca(1));
 
@@ -582,21 +744,8 @@ main(int argc, char **argv, char **envp) {
   ARGV = argv;
   ENVP = envp;
   
-  vs_top = vs_base = vs_org;
-  ihs_top = ihs_org-1;
-  bds_top = bds_org-1;
-  frs_top = frs_org-1;
-
   if (raw_image) {
 
-#ifdef GCL_GPROF
-    sigset_t prof;
-    int r;
-    sigemptyset(&prof);
-    sigaddset(&prof,SIGPROF);
-    sigprocmask(SIG_BLOCK,&prof,NULL);
-#endif
-
     printf("GCL (GNU Common Lisp)  %s  %ld pages\n",LISP_IMPLEMENTATION_VERSION,real_maxpage);
     fflush(stdout);
     
@@ -832,6 +981,7 @@ initlisp(void) {
 #ifdef USE_READLINE
 	gcl_init_readline();
 #endif
+	gcl_init_safety();
 
 }
 object
@@ -874,7 +1024,7 @@ ihs_overflow(void) {
 
 void
 segmentation_catcher(int i, long code, void *scp, char *addr) {
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(__CYGWIN__)
   void *faddr;
   faddr=GET_FAULT_ADDR(sig,code,scp,addr); 
 
@@ -975,7 +1125,9 @@ DEFUN("CATCH-FATAL",object,fScatch_fatal,SI,1,1,NONE,OI,OO,OO,OO,(fixnum i),"")
   return Cnil;
 }
 
-DEFUN("RESET-STACK-LIMITS",object,fSreset_stack_limits,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+DEFUN("RESET-STACK-LIMITS",object,fSreset_stack_limits,SI,0,1,NONE,OO,OO,OO,OO,(object cs_org_reset,...),"") {
+
+  if (!INIT_NARGS(0)) cs_org_reset=Cnil;
 
   if(catch_fatal <0) catch_fatal=1;
 #ifdef SGC	
@@ -997,15 +1149,23 @@ DEFUN("RESET-STACK-LIMITS",object,fSreset_stack_limits,SI,0,0,NONE,OO,OO,OO,OO,(
     ihs_limit = ihs_org + stack_multiple *  IHSSIZE;
   else
     error("can't reset ihs_limit");
-  cs_org=alloca(1);
+
+  if (cs_org_reset!=Cnil) {
+
+    cs_org=alloca(1);
+
 #ifdef __ia64__
- {
-   extern void * GC_save_regs_in_stack();
-   cs_org2=GC_save_regs_in_stack();
- }
+    {
+      extern void * GC_save_regs_in_stack();
+      cs_org2=GC_save_regs_in_stack();
+    }
 #endif
-  /* reset_cstack_limit(i); */
- RETURN1(Cnil);
+    /* reset_cstack_limit(i); */
+
+  }
+
+  RETURN1(Cnil);
+
 }
 
 #define COPYSTACK(org,p,typ,lim,top,geta,size) \
@@ -1027,15 +1187,15 @@ multiply_stacks(int m) {
   bd  = (stack_multiple*BDSSIZE + (STACK_OVER+1)*BDSGETA)*ELTSIZE(bds_org);
   frs = (stack_multiple*FRSSIZE + (STACK_OVER+1)*FRSGETA)*ELTSIZE(frs_org);
   ihs = (stack_multiple*IHSSIZE + (STACK_OVER+1)*IHSGETA)*ELTSIZE(ihs_org);
-  if (stack_space==0) {enter_mark_origin(&stack_space);}
-  stack_space = alloc_simple_string(vs+bd+frs+ihs);
-  array_allocself(stack_space,1,code_char(0));
-  p=stack_space->st.st_self;
+  if (stack_space==0)
+    enter_mark_origin(&stack_space);
+  massert(!((vs+bd+frs+ihs)%sizeof(fixnum)));
+  stack_space=fSmake_vector(make_fixnum(aet_fix),(vs+bd+frs+ihs)/sizeof(fixnum),Cnil,Cnil,Cnil,0,Ct,OBJNULL);
+  p=stack_space->v.v_self;
   COPYSTACK(vs_org,p,object,vs_limit,vs_top,VSGETA,VSSIZE);
   COPYSTACK(bds_org,p,struct bds_bd,bds_limit,bds_top,BDSGETA,BDSSIZE);
   COPYSTACK(frs_org,p,struct frame,frs_limit,frs_top,FRSGETA,FRSSIZE);
-  COPYSTACK(ihs_org,p,struct invocation_history,ihs_limit,ihs_top,
-	    IHSGETA,IHSSIZE);
+  COPYSTACK(ihs_org,p,struct invocation_history,ihs_limit,ihs_top,IHSGETA,IHSSIZE);
   vs_base=vs_top;
   return stack_multiple;
 }
@@ -1080,9 +1240,6 @@ DEFUN("IDENTITY",object,fLidentity,LISP,1,1,NONE,OO,OO,OO,OO,(object x0),"") {
   RETURN1 (x0);
 }
 
-DEFUN("GCL-COMPILE-TIME",object,fSgcl_compile_time,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
-  RETURN1 (make_simple_string(__DATE__ " " __TIME__));
-}
 
 DEFUN("LISP-IMPLEMENTATION-VERSION",object,fLlisp_implementation_version,LISP,0,0,NONE,OO,OO,OO,OO,(void),"") {
   RETURN1((make_simple_string(LISP_IMPLEMENTATION_VERSION)));
@@ -1259,7 +1416,7 @@ init_main(void) {
   
 }
 
-#ifdef HAVE_DIS_ASM_H
+#if defined(HAVE_DIS_ASM_H) && defined(OUTPUT_ARCH)
 
 #include "dis-asm.h"
 
@@ -1314,6 +1471,9 @@ DEFUN("DISASSEMBLE-INSTRUCTION",object,fSdisassemble_instruction,SI,1,1,NONE,OI,
       s(&i, stdout,(fprintf_ftype) my_fprintf,my_fprintf_styled);
       i.read_memory_func=my_read;
       i.print_address_func=my_pa;
+#if defined(OUTPUT_MACH)
+      i.mach=OUTPUT_MACH;
+#endif
       if ((s=dlsym(v,"disassembler"))) {
 	disassembler_ftype disasm=(disassembler_ftype)(ufixnum)s(OUTPUT_ARCH,false,0,NULL);/*bfd_mach_x86_64*/
 	bp=b;
