@@ -1,3 +1,4 @@
+/* Copyright (C) 2024 Camm Maguire */
 #include "include.h"
 #include "funlink.h"
 #include "page.h"
@@ -86,7 +87,12 @@ DEFUN("INIT-FUNCTION",object,fSinit_function,SI,7,7,NONE,OO,OO,OI,II, \
   d=data!=Cnil ? data : m;
   i=sSPinit;
   i=i ? i->s.s_dbind : i;
-  s=i && i!=OBJNULL && !get_pageinfo(addr) && ((void *)addr)>=data_start && type_of(addr)==t_fixnum ? i->v.v_self[fix(addr)] : addr;
+  if (is_text_addr(addr)||(get_pageinfo(addr)&&!is_bigger_fixnum(addr))||!i||i==OBJNULL)
+    s=addr;
+  else {
+    massert(type_of(addr)==t_fixnum);
+    s=i->v.v_self[fix(addr)];
+  }
   z=type_of(sc)==t_cons && sc->c.c_car==sSmacro; /*FIXME limited no. of args.*/
   sc=z ? sc->c.c_cdr : sc;
   sc=type_of(sc)==t_function ? sc->fun.fun_plist : sc;
