@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -672,13 +673,12 @@ LFD(Lexp)(void)
 	vs_base[0] = number_exp(vs_base[0]);
 }
 
-LFD(Lexpt)(void)
-{
-	check_arg(2);
-	check_type_number(&vs_base[0]);
-	check_type_number(&vs_base[1]);
-	vs_base[0] = number_expt(vs_base[0], vs_base[1]);
-	vs_popp;
+DEFUN("EXPT",object,fLexpt,LISP,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  check_type_number(&vs_base[0]);
+  check_type_number(&vs_base[1]);
+  RETURN1(number_expt(x,y));
+
 }
 
 LFD(Llog)(void)
@@ -765,87 +765,6 @@ FFN(siLmodf)(void)
 
 }
 
-#define CTMP(a_) (type_of(a_)==t_fixnum ? stoi(fix(a_)) : MP(a_))
-
-object 
-powm_bbb(object a,object i,object m) {
-  object ans=new_bignum();
-  if (type_of(i)==t_fixnum)
-    mpz_powm_ui(MP(ans),CTMP(a),fix(i),CTMP(m));
-  else
-    mpz_powm(MP(ans),CTMP(a),MP(i),CTMP(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_bfb(object a,fixnum i,object m) {
-  object ans=new_bignum();
-  mpz_powm_ui(MP(ans),CTMP(a),i,CTMP(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_fbb(fixnum a,object i,object m) {
-  object ans=new_bignum();
-  if (type_of(i)==t_fixnum)
-    mpz_powm_ui(MP(ans),stoi(a),fix(i),CTMP(m));
-  else
-    mpz_powm(MP(ans),stoi(a),MP(i),CTMP(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_ffb(fixnum a,fixnum i,object m) {
-  object ans=new_bignum();
-  mpz_powm_ui(MP(ans),stoi(a),i,CTMP(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_bbf(object a,object i,fixnum m) {
-  object ans=new_bignum();
-  if (type_of(i)==t_fixnum)
-    mpz_powm_ui(MP(ans),CTMP(a),fix(i),stoi(m));
-  else
-    mpz_powm(MP(ans),CTMP(a),MP(i),stoi(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_bff(object a,fixnum i,fixnum m) {
-  object ans=new_bignum();
-  mpz_powm_ui(MP(ans),CTMP(a),i,stoi(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_fbf(fixnum a,object i,fixnum m) {
-  object ans=new_bignum();
-  if (type_of(i)==t_fixnum)
-    mpz_powm_ui(MP(ans),stoi(a),fix(i),stoi(m));
-  else
-    mpz_powm(MP(ans),stoi(a),MP(i),stoi(m));
-  return(maybe_replace_big(ans));
-}
-
-object 
-powm_fff(fixnum a,fixnum i,fixnum m) {
-  object ans=new_bignum();
-  mpz_powm_ui(MP(ans),stoi(a),i,stoi(m));
-  return(maybe_replace_big(ans));
-}
-
-
-DEFUN("POWM",object,fSpowm,SI,3,3,NONE,OO,OO,OO,OO,(object a,object i,object m),"") {
-
-  check_type_integer(&a);
-  check_type_integer(&i);
-  check_type_integer(&m);
-  RETURN1(powm_bbb(a,i,m));
-
-}
-
-
 DEFUN("ISNAN",object,fSisnan,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
   switch (type_of(x)) {
@@ -903,7 +822,6 @@ gcl_init_num_sfun(void)
 	make_constant("PI", make_longfloat(PI));
 
 	make_function("EXP", Lexp);
-	make_function("EXPT", Lexpt);
 	make_function("LOG", Llog);
 	make_function("SQRT", Lsqrt);
 	make_function("SIN", Lsin);
