@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -56,12 +57,13 @@ object small_fixnum ( int i ) {
 #endif
 
 
+/*FIXME, make these immutable and of type immfix*/
 #define BIGGER_FIXNUM_RANGE
 
 #ifdef BIGGER_FIXNUM_RANGE
 struct {int min,max;} bigger_fixnums;
 
-struct fixnum_struct *bigger_fixnum_table;
+struct fixnum_struct *bigger_fixnum_table=NULL,*bigger_fixnum_table_end=NULL;
 #if !defined(IM_FIX_BASE) || defined(USE_SAFE_CDR)
 #define STATIC_BIGGER_FIXNUM_TABLE_BITS 10
 static struct fixnum_struct bigger_fixnum_table1[1<<(STATIC_BIGGER_FIXNUM_TABLE_BITS+1)] OBJ_ALIGN;
@@ -74,11 +76,15 @@ DEFUN("ALLOCATE-BIGGER-FIXNUM-RANGE",object,fSallocate_bigger_fixnum_range,SI,2,
   if (min > max) FEerror("Need Min <= Max",0);
 
 #if !defined(IM_FIX_BASE) || defined(USE_SAFE_CDR)
-  if (min==-(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS) && max==(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS))
+  if (min==-(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS) && max==(1<<STATIC_BIGGER_FIXNUM_TABLE_BITS)) {
     bigger_fixnum_table=bigger_fixnum_table1;
-  else
+    bigger_fixnum_table_end=(void *)bigger_fixnum_table+sizeof(bigger_fixnum_table1);
+  } else
 #endif
-    bigger_fixnum_table=(void *)malloc(sizeof(struct fixnum_struct)*(max - min));
+    {
+      bigger_fixnum_table=(void *)malloc(sizeof(struct fixnum_struct)*(max - min));
+      bigger_fixnum_table_end=(void *)bigger_fixnum_table+sizeof(struct fixnum_struct)*(max - min);
+    }
   
   for (j=min ; j < max ; j=j+1) { 		
     object x=(object)(bigger_fixnum_table+j-min);
@@ -93,6 +99,11 @@ DEFUN("ALLOCATE-BIGGER-FIXNUM-RANGE",object,fSallocate_bigger_fixnum_range,SI,2,
 }
 #endif
 
+int
+is_bigger_fixnum(void *v)  {
+  return v>=(void *)bigger_fixnum_table && v<(void *)bigger_fixnum_table_end ? 1 : 0;
+}
+
 object
 make_fixnum1(long i)
 {
