@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 W. Schelter
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -56,11 +57,20 @@ License for more details.
 #define ALLOC_SEC(sec) (sec->sh_flags&SHF_ALLOC && (sec->sh_type==SHT_PROGBITS || sec->sh_type==SHT_NOBITS))
 #define LOAD_SEC(sec) (sec->sh_flags&SHF_ALLOC &&  sec->sh_type==SHT_PROGBITS)
 #define LOAD_SYM(sym,st1) (sym->st_value && (EXT_SYM(sym,st1)||LOCAL_SYM(sym)))
-#define EXT_SYM(sym,st1) (ELF_ST_BIND(sym->st_info)==STB_GLOBAL||ELF_ST_BIND(sym->st_info)==STB_WEAK||LOAD_SYM_BY_NAME(sym,st1))
 #define LOCAL_SYM(sym) ELF_ST_BIND(sym->st_info)==STB_LOCAL
-#define LOAD_SYM_BY_NAME(sym,st1) !strncmp(st1+sym->st_name,"__muldc3",8)||!strncmp(st1+sym->st_name,"__divdc3",8)
+#define EXT_SYM(sym,st1) (ELF_ST_BIND(sym->st_info)==STB_GLOBAL||	\
+			  ELF_ST_BIND(sym->st_info)==STB_WEAK||		\
+			  GCC_SYM(sym,st1)||				\
+			  PSPEC_SYM(sym,st1))
+/*e.g. arm soft float functions*/
+#define GCC_SYM(sym,st1) (ELF_ST_BIND(sym->st_info)==STB_LOCAL &&	\
+			  ELF_ST_TYPE(sym->st_info)==STT_FUNC &&	\
+			  st1[sym->st_name]=='_')
+/*e.g. parisc millicode*/
+#define PSPEC_SYM(sym,st1) (ELF_ST_BIND(sym->st_info)==STB_LOCAL &&	\
+			    ELF_ST_TYPE(sym->st_info)>=STT_LOPROC)
 
-#define MASK(n) (~(~0ULL << (n)))
+#define MASK(n) ((ul)(~(~0ULL << (n))))
 
 
 
@@ -318,6 +328,8 @@ load_memory(Shdr *sec1,Shdr *sece,void *v1,ul **got,ul **gote) {
       sec->sh_addr+=a;
       if (LOAD_SEC(sec))
 	memcpy((void *)sec->sh_addr,v1+sec->sh_offset,sec->sh_size);
+      else
+	bzero((void *)sec->sh_addr,sec->sh_size);
     }
   
   if (**got) {
