@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 Rami el Charif, W. Schelter 
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -433,6 +434,7 @@ fill pointer, and this will be advanced.")
     if (x->s.s_hpack == keyword_package)
       {if (code == normal)
 	 PUSH('-');}
+    x=x->s.s_name;
   case t_simple_string:/*FIXME?*/
   case t_string:
     {int len = VLEN(x);
@@ -528,10 +530,10 @@ and positioning the ouput/input to start at START and end at END")
   return strm;
 }
 
-DEFUN("CHECK-STATE-INPUT",object,fScheck_state_input,SI,2,2,NONE,OO,IO,OO,OO,(object osfd,fixnum timeout),
+DEFUN("CHECK-STATE-INPUT",object,fScheck_state_input,SI,2,2,NONE,IO,IO,OO,OO,(object osfd,fixnum timeout),
       "") 
 {
-  return fScheck_dsfd_for_input(OBJ_TO_CONNECTION_STATE(osfd),timeout);
+  return (object)fScheck_dsfd_for_input(OBJ_TO_CONNECTION_STATE(osfd),timeout);
 
 }
 
