@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -24,6 +25,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <spawn.h>
 #ifndef __MINGW32__
 #include <sys/wait.h>
 #endif
@@ -32,14 +34,17 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #if !defined(__MINGW32__) && !defined(__CYGWIN__)
 
-int
-vsystem(const char *command) {
 
-  unsigned j,n=strlen(command)+1;
-  char *z,*c;
-  const char *x1[]={"/bin/sh","-c",NULL,NULL},*spc=" \n\t",**p1,**pp;
+int
+vsystem(char *command) {
+
+  char *c;
+  const char *x1[]={"/bin/sh","-c",NULL,NULL},*spc=" \n\t",**p1,**pp,**pe;
   int s;
   pid_t pid;
+  posix_spawnattr_t attr;
+  posix_spawn_file_actions_t file_actions;
+  extern char **environ;
 
   if (strpbrk(command,"\"'$<>"))
 
@@ -47,22 +52,20 @@ vsystem(const char *command) {
 
   else {
 
-    massert(n<sizeof(FN1));
-    memcpy((z=FN1),command,n);
-    for (j=1,c=z;strtok(c,spc);c=NULL,j++);
-
-    memcpy(z,command,n);
-    massert(j*sizeof(*p1)<sizeof(FN2));
     p1=(void *)FN2;
-    for (pp=p1,c=z;(*pp=strtok(c,spc));c=NULL,pp++);
+    pe=p1+sizeof(FN2)/sizeof(*p1);
+    for (pp=p1,c=command;pp<pe && (*pp=strtok(c,spc));c=NULL,pp++);
+    massert(pp<pe);
 
   }
 
-  if (!(pid=pvfork())) {
-    errno=0;
-    execvp(*p1,(void *)p1);
-    _exit(128|(errno&0x7f));
-  }
+  massert(!posix_spawn_file_actions_init(&file_actions));
+  massert(!posix_spawnattr_init(&attr));
+
+  massert(!posix_spawnp(&pid, *p1, &file_actions, &attr,  (void *)p1, environ));
+
+  massert(!posix_spawnattr_destroy(&attr));
+  massert(!posix_spawn_file_actions_destroy(&file_actions));
 
   massert(pid>0);
   massert(pid==waitpid(pid,&s,0));
@@ -145,7 +148,7 @@ char *command;
 #endif
 
 int
-msystem(const char *s) {
+msystem(char *s) {
 
   return psystem(s);
 
@@ -154,7 +157,7 @@ msystem(const char *s) {
 static void
 FFN(siLsystem)(void)
 {
-	char command[32768];
+	static char command[32768];
 	int i;
 
 	check_arg(1);
