@@ -2,6 +2,7 @@
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
+;;; Copyright (c) 2024 Camm Maguire
 ;;; All rights reserved.
 ;;;
 ;;; Use and copying of this software and preparation of derivative works
@@ -627,15 +628,15 @@ work during bootstrapping.
          (bind-args (,lambda-list ,method-args)
 	   ,@body)))))
 
-(defmacro fast-lexical-method-functions 
-  ((lambda-list next-method-call args rest-arg &rest lmf-options)
-   &body body)
- `(bind-fast-lexical-method-macros (,args ,rest-arg ,next-method-call)
-    (bind-lexical-method-functions (,@lmf-options)
-      (let ,(mapcar (lambda (x) (list x x)) args)
-	(declare (ignorable ,@args))
-        (bind-args (,(nthcdr (length args) lambda-list) ,rest-arg)
-		   ,@body)))))
+(defmacro fast-lexical-method-functions ((lambda-list next-method-call args rest-arg &rest lmf-options)
+					 &body body)
+  `(bind-fast-lexical-method-macros
+    (,args ,rest-arg ,next-method-call)
+    (bind-lexical-method-functions
+     (,@lmf-options)
+     (bind-args
+      (,lambda-list (list* ,@args ,rest-arg))
+      ,@body))))
 
 (defun call-no-next-method (method-name-declaration &rest args)
   (destructuring-bind (name qualifiers specializers)
@@ -643,14 +644,6 @@ work during bootstrapping.
     (let ((method (find-method (gdefinition name) qualifiers specializers)))
       (apply #'no-next-method (method-generic-function method) method args))))
 
-(defun generic-function-nreq (gf)
-  (let* ((arg-info (if (early-gf-p gf)
-                       (early-gf-arg-info gf)
-                       (gf-arg-info gf)));safe
-         (metatypes (arg-info-metatypes arg-info)))
-    (declare (list metatypes))
-    (length metatypes)))
-
 (defun %check-cnm-args (cnm-args orig-args method-name-declaration)
   (declare (optimize (speed 3) (safety 0) (debug 0))
            (type list cnm-args orig-args))
@@ -948,7 +941,7 @@ work during bootstrapping.
 	       (declare (dynamic-extent cnm-args))
 	       (call-next-method-body ,method-name-declaration cnm-args))
 	     (next-method-p () (next-method-p-body)))
-;	(declare (ignorable #'call-next-method #'next-method-p))
+	(declare (ignorable #'call-next-method #'next-method-p))
 	,@body)))
 
 (defmacro bind-args ((lambda-list args) &body body)
@@ -1334,6 +1327,14 @@ work during bootstrapping.
        (unless (equal ,pos ,valsym)
 	 (setf ,pos ,valsym)))))
 
+(defun generic-function-nreq (gf)
+  (let* ((arg-info (if (early-gf-p gf)
+                       (early-gf-arg-info gf)
+                       (gf-arg-info gf)));safe
+         (metatypes (arg-info-metatypes arg-info)))
+    (declare (list metatypes))
+    (length metatypes)))
+
 (defun set-arg-info (gf &key new-method (lambda-list nil lambda-list-p)
 			argument-precedence-order)
   (let* ((arg-info (if (eq *boot-state* 'complete)
@@ -2120,7 +2121,7 @@ work during bootstrapping.
 		 ,gensyms 
 		 ,(caddr form)
 		 .,(reverse (mapcar (lambda (v g) `(setf ,v ,g)) vars gensyms))))))
-	  ((eq (car form) 'conditions::restart-case)
+	  ((eq (car form) 'restart-case)
 	   (let* ((nf (cadr form))
 		  (nf (let ((y (assoc nf specs)))
 			(if (and y (eq (cadr y) (variable-lexical-p nf env))) (walker::macroexpand-all (caddr y) env) nf))))
