@@ -2,6 +2,7 @@
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
+;;; Copyright (c) 2024 Camm Maguire
 ;;; All rights reserved.
 ;;;
 ;;; Use and copying of this software and preparation of derivative works
@@ -129,6 +130,7 @@
 
 (eval-when (compile load eval)
   (defun extract-declarations (body &optional environment)
+    (declare (ignore environment))
     (multiple-value-bind (doc decls ctps body) (si::parse-body-header body)
       (values doc decls (nconc ctps body)))))
 
@@ -319,9 +321,9 @@
 ;;; Similar to printing-random-object in the lisp machine but much simpler
 ;;; and machine independent.
 (defmacro printing-random-thing ((thing stream) &body body)
-  #+cmu17
+  #+(or cmu17 gcl)
   `(print-unreadable-object (,thing ,stream :identity t) ,@body)
-  #-cmu17
+  #-(or cmu17 gcl)
   (once-only (thing stream)
     `(progn
        #+cmu
