@@ -2,6 +2,7 @@
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
+;;; Copyright (c) 2024 Camm Maguire
 ;;; All rights reserved.
 ;;;
 ;;; Use and copying of this software and preparation of derivative works
@@ -397,11 +398,12 @@
 		(loop for (option value . more) on slot by #'cddr
 		      when (and (member option '(:allocation :type :initform
 							     :documentation))
+				(progn value t)
 				(not (eq unsupplied
-					 (getf more option unsupplied)))) do
-					 (error 'program-error :format-control
-							 "Duplicate slot option ~S for slot ~S."
-							 :format-arguments (list option slot-name))))
+					 (getf more option unsupplied))))
+		      do (error 'program-error :format-control
+				"Duplicate slot option ~S for slot ~S."
+				:format-arguments (list option slot-name))))
 	  ;;
 	  ;; CLHS: signal PROGRAM-ERROR, if an initialization argument name
 	  ;; appears more than once in :DEFAULT-INITARGS class option.
