@@ -52,6 +52,7 @@
 ;;; 
 
 (in-package :walker)
+#+gcl(import 'si::macro)
 
 ;;;
 ;;; The user entry points are walk-form and nested-walked-form.  In addition,
@@ -61,6 +62,7 @@
 ;;; out for themselves.
 ;;; 
 (export '(define-walker-template
+	  #+gcl macro
 	  walk-form
 	  walk-form-expand-macros-p
 	  nested-walk-form
@@ -608,7 +610,7 @@
       (push `(,(car f) .  (function  . (,#'unbound-lexical-function . nil)))
 	    lexicals))
     (dolist (m macros)
-      (push `(,(car m)  .  (si::macro . ( ,(cadr m) . nil))) 
+      (push `(,(car m)  .  (macro . ( ,(cadr m) . nil))) 
 	    lexicals))
     (list first lexicals third)))
 
@@ -623,7 +625,7 @@
   (when env
 	(let ((entry (assoc macro (second env))))
 	  (and entry
-	       (eq (cadr entry) 'si::macro)
+	       (eq (cadr entry) 'macro)
 	       (caddr entry)))))
 );#+(or KCL IBCL)
 
@@ -1202,7 +1204,7 @@
 
 #+(or KCL IBCL)
 (progn
-  (define-walker-template si::lambda-block walk-named-lambda);Not really right,
+  (define-walker-template lambda-block walk-named-lambda);Not really right,
 							 ;we don't hack block
 						         ;names anyways.
   )
